/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.interfaces;

import org.eclipse.emf.ecore.EObject;

/**
 * Interface for connectors, i.e. classes which connect model elements. This may
 * either be a direct connection or by using an additional connection element.
 * 
 * @TODO: rename to "ConnectionCompositor" + make the parent explicit
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 3AF8EF4B6F61B768A17A63A6FF51CB2B
 */
public interface IConnectionCompositor<S extends EObject, T extends EObject> extends
		IEObjectAware2<S, T> {
	/**
	 * Returns whether a connection between the source and the destination is
	 * possible. Additional information can be included in the context, which
	 * also may be null. The connection parameter holds an existing connection
	 * in case of a reconnect.
	 */
	boolean canConnect(S source, T destination, EObject connection,
			IConnectionCompositionContext context);

	/**
	 * Actually connects the given source and destination. This will only be
	 * called if
	 * {@link #canConnect(EObject, EObject, EObject, IConnectionCompositionContext)}
	 * returned true.
	 * 
	 * @param source
	 *            the source of the connection.
	 * @param destination
	 *            the destination of the connection.
	 * @param connection
	 *            the (existing) connection object. This may be null to indicate
	 *            that no connection object exists yet, or none is needed
	 *            (direct connection). If this is not null, this should be
	 *            reused for the connection (reconnect).
	 * @param context
	 *            context information for the connection. May be null.
	 */
	void connect(S source, T destination, EObject connection,
			IConnectionCompositionContext context);
}
