/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.base.EObjectAware2ServiceBase;
import org.fortiss.tooling.kernel.interfaces.IConnectionCompositionContext;
import org.fortiss.tooling.kernel.interfaces.IConnectionCompositor;
import org.fortiss.tooling.kernel.services.IConnectionCompositorService;

/**
 * This class implements the {@link IConnectionCompositorService} interface.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: E4E7A3B42928271F17D36BABCB6BEF58
 */
public class ConnectionCompositorService extends
		EObjectAware2ServiceBase<IConnectionCompositor<EObject, EObject>>
		implements IConnectionCompositorService {

	/** The connector extension point ID. */
	private static final String EXTENSION_POINT_NAME = "org.fortiss.tooling.kernel.modelElementConnectionCompositor";

	/** The connector configuration element name. */
	private static final String CONFIGURATION_ELEMENT_NAME = "modelElementConnectionCompositor";

	/** The connector attribute name. */
	private static final String ATTRIBUTE_NAME = "modelElementConnectionCompositor";

	/** {@inheritDoc} */
	@Override
	public boolean canConnect(EObject source, EObject destination,
			EObject connection, IConnectionCompositionContext context) {
		return findWorkingConnector(source, destination, connection, context) != null;
	}

	/** {@inheritDoc} */
	@Override
	public void connect(EObject source, EObject destination,
			EObject connection, IConnectionCompositionContext context) {
		findWorkingConnector(source, destination, connection, context).connect(
				source, destination, connection, context);
	}

	/**
	 * Returns the first {@link IConnectionCompositor} which can connect the
	 * given elements (or <code>null</code>n if no such connector exists).
	 */
	private IConnectionCompositor<EObject, EObject> findWorkingConnector(
			EObject source, EObject target, EObject connection,
			IConnectionCompositionContext context) {
		List<IConnectionCompositor<EObject, EObject>> list = getRegisteredHandlers(
				source.getClass(), target.getClass());
		if (list == null) {
			return null;
		}
		for (IConnectionCompositor<EObject, EObject> connector : list) {
			if (connector.canConnect(source, target, connection, context)) {
				return connector;
			}
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected String getExtensionPointName() {
		return EXTENSION_POINT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getConfigurationElementName() {
		return CONFIGURATION_ELEMENT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getHandlerClassAttribute() {
		return ATTRIBUTE_NAME;
	}
}
